#include "Arduino.h"

#include "TM1638_ESP32.h"

TM1638_ESP32::TM1638_ESP32(uint8_t strobe, uint8_t clock, uint8_t data, uint8_t clock_type, uint8_t clock_delay_us) {
  STROBE_IO = strobe;
  CLOCK_IO = clock;
  DATA_IO = data;
  CLOCK_TYPE = clock_type;
  CLOCK_DELAY_US = clock_delay_us;
  
  pinMode(strobe, OUTPUT);
  pinMode(clock, OUTPUT);
  pinMode(data, OUTPUT);
  
  sendCommand(TM1638_ACTIVATE);
  reset();
}

void TM1638_ESP32::sendCommand(uint8_t value)
{
  digitalWrite(STROBE_IO, LOW);
  shiftOutMod(DATA_IO, CLOCK_IO, LSBFIRST, value);
  digitalWrite(STROBE_IO, HIGH);
}

void TM1638_ESP32::reset()
{
  sendCommand(TM1638_WRITE_INC); // set auto increment mode
  digitalWrite(STROBE_IO, LOW);
  shiftOutMod(DATA_IO, CLOCK_IO, LSBFIRST, 0xc0);   // set starting address to 0
  for (uint8_t i = 0; i < 16; i++)
  {
    shiftOutMod(DATA_IO, CLOCK_IO, LSBFIRST, 0x00);
  }
  digitalWrite(STROBE_IO, HIGH);
}

uint8_t TM1638_ESP32::readButtons()
{
  uint8_t buttons = 0;
  digitalWrite(STROBE_IO, LOW);
  shiftOutMod(DATA_IO, CLOCK_IO, LSBFIRST, TM1638_BUTTONS);
  pinMode(DATA_IO, INPUT);

  for (uint8_t i = 0; i < 4; i++)
  {
    uint8_t v = shiftInMod(DATA_IO, CLOCK_IO, LSBFIRST) << i;
    buttons |= v;
  }

  pinMode(DATA_IO, OUTPUT);
  digitalWrite(STROBE_IO, HIGH);
  return buttons;
}

void TM1638_ESP32::setLED(uint8_t position, uint8_t value)
{
  pinMode(DATA_IO, OUTPUT);
  sendCommand(TM1638_WRITE_LOC);
  digitalWrite(STROBE_IO, LOW);
  shiftOutMod(DATA_IO, CLOCK_IO, LSBFIRST, 0xC1 + (position << 1));
  shiftOutMod(DATA_IO, CLOCK_IO, LSBFIRST, value);
  digitalWrite(STROBE_IO, HIGH);
}

void TM1638_ESP32::displayText(String text) {
  uint8_t length = text.length();

  for (uint8_t i = 0; i < length; i++) {
    for (uint8_t position = 0; position < 8; position++) {
      displayASCII(position, text[position]);
    }
  }
}

void TM1638_ESP32::displaySS(uint8_t position, uint8_t value) { // call 7-segment
  sendCommand(TM1638_WRITE_LOC);
  digitalWrite(STROBE_IO, LOW);
  shiftOutMod(DATA_IO, CLOCK_IO, LSBFIRST, 0xC0 + (position << 1));
  shiftOutMod(DATA_IO, CLOCK_IO, LSBFIRST, value);
  digitalWrite(STROBE_IO, HIGH);
}

void TM1638_ESP32::displayASCII(uint8_t position, uint8_t ascii) {
  displaySS(position, ss[ascii]);
}

void TM1638_ESP32::displayHex(uint8_t position, uint8_t hex) {
  displaySS(position, hexss[hex]);
}

void TM1638_ESP32::shiftOutMod(uint8_t dataPin, uint8_t clockPin, uint8_t bitOrder, uint8_t val)
{
	uint8_t i;

	for (i = 0; i < 8; i++)  {
		if (bitOrder == LSBFIRST)
			digitalWrite(dataPin, !!(val & (1 << i)));
		else	
			digitalWrite(dataPin, !!(val & (1 << (7 - i))));
			
		digitalWrite(clockPin, (CLOCK_TYPE ? LOW : HIGH));
    delayMicroseconds(CLOCK_DELAY_US);
		digitalWrite(clockPin, (CLOCK_TYPE ? HIGH : LOW));		
	}
}

uint8_t TM1638_ESP32::shiftInMod(uint8_t dataPin, uint8_t clockPin, uint8_t bitOrder) {
	uint8_t value = 0;
	uint8_t i;

  
	for (i = 0; i < 8; ++i) {
		digitalWrite(clockPin, (CLOCK_TYPE ? LOW : HIGH));
    delayMicroseconds(CLOCK_DELAY_US);
		if (bitOrder == LSBFIRST)
			value |= digitalRead(dataPin) << i;
		else
			value |= digitalRead(dataPin) << (7 - i);
		digitalWrite(clockPin, (CLOCK_TYPE ? HIGH : LOW));
	}
	return value;
}

