#include "TM1638_ESP32.h"

const int strobe_pin =  4;
const int clock_pin  = 16;
const int data_pin   = 17;

// TM1638_ESP32(Strobe, Clock, Data)
TM1638_ESP32 tm(strobe_pin, clock_pin, data_pin);

#define COUNTING_MODE 0
#define SCROLL_MODE 1
#define BUTTON_MODE 2

void setup()
{
}

void loop()
{
  static uint8_t mode = COUNTING_MODE;

  switch(mode)
  {
  case COUNTING_MODE:
    mode += counting();
    break;
  case SCROLL_MODE:
    mode += scroll();
    break;
  case BUTTON_MODE:
    buttons();
    break;
  }

  delay(200);
}

bool counting()
{
                       /*0*/ /*1*/ /*2*/ /*3*/ /*4*/ /*5*/ /*6*/ /*7*/ /*8*/ /*9*/
  uint8_t digits[] = { 0x3f, 0x06, 0x5b, 0x4f, 0x66, 0x6d, 0x7d, 0x07, 0x7f, 0x6f };

  static uint8_t digit = 0;

  tm.sendCommand(WRITE_INC);
  digitalWrite(strobe_pin, LOW);
  tm.shiftOutMod(data_pin, clock_pin, LSBFIRST, 0xc0);
  for(uint8_t position = 0; position < 8; position++)
  {
    tm.shiftOutMod(data_pin, clock_pin, LSBFIRST, digits[digit]);
    tm.shiftOutMod(data_pin, clock_pin, LSBFIRST, 0x00);
  }

  digitalWrite(strobe_pin, HIGH);

  digit = ++digit % 10;
  return digit == 0;
}

bool scroll()
{
  uint8_t scrollText[] =
  {
    /* */ /* */ /* */ /* */ /* */ /* */ /* */ /* */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    /*H*/ /*E*/ /*L*/ /*L*/ /*O*/ /*.*/ /*.*/ /*.*/
    0x76, 0x79, 0x38, 0x38, 0x3f, 0x80, 0x80, 0x80,
    /* */ /* */ /* */ /* */ /* */ /* */ /* */ /* */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    /*H*/ /*E*/ /*L*/ /*L*/ /*O*/ /*.*/ /*.*/ /*.*/
    0x76, 0x79, 0x38, 0x38, 0x3f, 0x80, 0x80, 0x80,
  };

  static uint8_t index = 0;
  uint8_t scrollLength = sizeof(scrollText);

  tm.sendCommand(WRITE_INC);
  digitalWrite(strobe_pin, LOW);
  tm.shiftOutMod(data_pin, clock_pin, LSBFIRST, 0xc0);

  for(int i = 0; i < 8; i++)
  {
    uint8_t c = scrollText[(index + i) % scrollLength];

    tm.shiftOutMod(data_pin, clock_pin, LSBFIRST, c);
    tm.shiftOutMod(data_pin, clock_pin, LSBFIRST, c != 0 ? 1 : 0);
  }

  digitalWrite(strobe_pin, HIGH);

  index = ++index % (scrollLength << 1);

  return index == 0;
}

void buttons()
{
  uint8_t promptText[] =
  {
    /*P*/ /*r*/ /*E*/ /*S*/ /*S*/ /* */ /* */ /* */
    0x73, 0x50, 0x79, 0x6d, 0x6d, 0x00, 0x00, 0x00,
    /* */ /* */ /* */ /* */ /* */ /* */ /* */ /* */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    /*b*/ /*u*/ /*t*/ /*t*/ /*o*/ /*n*/ /*S*/ /* */
    0x7c, 0x1c, 0x78, 0x78, 0x5c, 0x54, 0x6d, 0x00,
    /* */ /* */ /* */ /* */ /* */ /* */ /* */ /* */
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  };

  static uint8_t block = 0;

  uint8_t textStartPos = (block / 4) << 3;
  for(uint8_t position = 0; position < 8; position++)
  {
    tm.sendCommand(WRITE_LOC);
    digitalWrite(strobe_pin, LOW);
    tm.shiftOutMod(data_pin, clock_pin, LSBFIRST, 0xC0 + (position << 1));
    tm.shiftOutMod(data_pin, clock_pin, LSBFIRST, promptText[textStartPos + position]);
    digitalWrite(strobe_pin, HIGH);
  }

  block = (block + 1) % 16;

  uint8_t buttons = tm.readButtons();

  for(uint8_t position = 0; position < 8; position++)
  {
    uint8_t mask = 0x1 << position;

    tm.setLED(position, buttons & mask ? 1 : 0);
  }
}
